/*
* Copyright (c) 2005, Bear Bibeault
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*  - The name of Bear Bibeault may not be used to endorse or promote products
*    derived from this software without specific prior written permission.
*
* This software is provided by the copyright holders and contributors "as is"
* and any express or implied warranties, including, but not limited to, the
* implied warranties of merchantability and fitness for a particular purpose
* are disclaimed. In no event shall the copyright owner or contributors be
* liable for any direct, indirect, incidental, special, exemplary, or
* consequential damages (including, but not limited to, procurement of
* substitute goods or services; loss of use, data, or profits; or business
* interruption) however caused and on any theory of liability, whether in
* contract, strict liability, or tort (including negligence or otherwise)
* arising in any way out of the use of this software, even if advised of the
* possibility of such damage.
*/
package org.bibeault.jrjournal.ccc;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

/**
 * Service class that looks up the value of class constants. This class
 * considers "class constant" to mean a field with modifiers: public, static
 * and final.
 *
 * @author <a href="mailto:bear@javaranch.com">Bear Bibeault</a>
 */
public class ClassConstantInspector {

    private Field field;

    /**
     * Constructs a class constant inspector for the passed field. The
     * field is checked to ensure that it is a public static final field.
     *
     * @param constantPath the fully qualified name of the class class constant
     *
     * @throws ClassNotFoundException if the named class does not exist
     * @throws NoSuchFieldException if the named field does not exists
     * @throws IllegalArgumentException if the class or field fails to meet the constrictions described above
     */
    public ClassConstantInspector( String constantPath ) throws ClassNotFoundException, NoSuchFieldException {
        FieldPathParser parser = new FieldPathParser( constantPath );
        this.field = Class.forName( parser.getDeclaringClassName() ).getField( parser.getFieldName() );
        if (!Modifier.isPublic( this.field.getModifiers() ) ||
            !Modifier.isStatic( this.field.getModifiers() ) ||
            !Modifier.isFinal( this.field.getModifiers() ) ) {
            throw new IllegalArgumentException( "Field " + constantPath + " is not a public static final field" );
        }
    }

    /**
     * Returns the value of the inspected class constant.
     *
     * @return the value of the inspected constant
     * @throws IllegalAccessException if the field cannot be accessed
     * @throws InstantiationException if the class cannot be instantiated
     */
    public Object getValue() throws IllegalAccessException, InstantiationException {
        return this.field.get( null );
    }

    /**
     * Static convenience method to obtain the value of a constant. Suitable for
     * use as an EL function.
     *
     * @param constantName the fully qualified name of the class class constant
     *
     * @return the value of the inspected constant
     * @throws NoSuchFieldException if the field does not exist
     * @throws ClassNotFoundException if the class does not exist
     * @throws IllegalAccessException if the field cannot be accessed
     * @throws InstantiationException if the class cannot be instantiated
     */
    public static Object getValue( String constantName ) throws NoSuchFieldException, ClassNotFoundException, IllegalAccessException, InstantiationException {
        return new ClassConstantInspector( constantName ).getValue();
    }

}
