/*
* Copyright (c) 2005, Bear Bibeault
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*  - The name of Bear Bibeault may not be used to endorse or promote products
*    derived from this software without specific prior written permission.
*
* This software is provided by the copyright holders and contributors "as is"
* and any express or implied warranties, including, but not limited to, the
* implied warranties of merchantability and fitness for a particular purpose
* are disclaimed. In no event shall the copyright owner or contributors be
* liable for any direct, indirect, incidental, special, exemplary, or
* consequential damages (including, but not limited to, procurement of
* substitute goods or services; loss of use, data, or profits; or business
* interruption) however caused and on any theory of liability, whether in
* contract, strict liability, or tort (including negligence or otherwise)
* arising in any way out of the use of this software, even if advised of the
* possibility of such damage.
*/
package org.bibeault.jrjournal.ccc;

import javax.servlet.ServletContextListener;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContext;

/**
 * Context listener that establishes an instance of {@link ClassConstantMapper} in
 * application scope. The name of the scoped variable is provided by a
 * context parameter named by the <code>PARAM_CONSTANT_MAPPER_NAME</code>
 * class constant whose value is
 * <code>org.bibeault.jrjournal.ccc.ClassConstantMapperEstablisher.classConstantMapperName</code>
 *
 * <p>An example usage in a deployment descriptor (web.xml) might be:
 * <pre>
&lt;context-param&gt;
  &lt;param-name&gt;org.bibeault.jrjournal.ccc.ClassConstantMapperEstablisher.classConstantMapperName&lt;/param-name&gt;
  &lt;param-value&gt;constants&lt;/param-value&gt;
&lt;/context-param&gt;

&lt;listener&gt;
  &lt;listener-class&gt;org.bibeault.jrjournal.ccc.ClassConstantMapperEstablisher&lt;/listener-class&gt;
&lt;/listener&gt;
 * </pre>
 * </p>
 * @author <a href="mailto:bear@javaranch.com">Bear Bibeault</a>
 */
public class ClassConstantMapperEstablisher implements ServletContextListener {

    public static final String PARAM_CONSTANT_MAPPER_NAME = ClassConstantMapperEstablisher.class.getName() + ".classConstantMapperName";

    /**
     * Activated by the container when the application is placed into service
     * in order to establish an instance of {@link ClassConstantMapper} in
     * application context.
     *
     * <p>The scoped variable is created with a name as specified by the
     * <code>org.bibeault.jrjournal.ccc.ClassConstantMapperEstablisher.classConstantMapperName</code>
     * context parameter in the deployment descriptor.
     *
     * @param servletContextEvent the servlet context event
     */
    public void contextInitialized( ServletContextEvent servletContextEvent ) {
        ServletContext servletContext = servletContextEvent.getServletContext();
        String varName = servletContext.getInitParameter( PARAM_CONSTANT_MAPPER_NAME );
        if (varName == null) throw new IllegalStateException( "Context parameter " + PARAM_CONSTANT_MAPPER_NAME + " must be specified" );
        servletContext.setAttribute( varName, new ClassConstantMapper() );
    }

    public void contextDestroyed( ServletContextEvent servletContextEvent ) {}

}
